#---------------------------------------------------------
# Scriptname: 1.from_cases_to_case_crossover.R
#
# Purpose of script:
#
# Create a case-crossover dataset from list of cases and
# merge the case-crossover dataset with meteorological 
# factors lagged up to 8 weeks.
#         
#  ------------------------------------------------------
#  ! This script is run on simulated data:              !
#  !                                                    !
#  ! - "sim_data.csv"  is a simulated dataset including !
#  !    cases from 4 different geographical areas       !
#  !    (NUTS_ID).                                      !
#  ------------------------------------------------------
#
# Author(s): Giovenale Moirano
# Date Created: 05 May 2024
# Email: giovenale.moirano@bsc.es
#
#---------------------------------------------------------
# 1. Clean environment & set directory.
#---------------------------------------------------------

rm(list = ls())

#------------------------------------------------------------
# 2.Set WD
#------------------------------------------------------------

# setwd("mydirectory/")

#---------------------------------------------------------
# 3. Loadpackages.
#---------------------------------------------------------

packages <- c(
  "ggmap", "sf", "zoo", "lubridate",
  "dplyr", "terra", "readxl", "Hmisc"
  )

install.packages(setdiff(packages, rownames(installed.packages())))
lapply(packages, require, character.only = TRUE)

# Source function that create a case crossover dataset from
# lines of cases

source("functions/tscc_sampling.R")


#---------------------------------------------------------
# 4. Load Simulated Cases.
#---------------------------------------------------------

data <- read.csv(file = "data/3.sim_cases.csv")
head(data)

#------------------------------------------------------------
# 5. Create a case-crossover dataset.
#------------------------------------------------------------

data$ID <- paste0("ID", seq(1 : length(data$date)))
head(data)

# 5a. Create a space-time-stratified Case Crossover dataset with the
# 'tscc_sampling' function.

db_cc <- tscc_sampling(data = data, dates = "date")
head(db_cc)

# 5b. Merge the case-crossover dataset with NUTS_ID of orginal cases.
data <- db_cc %>%
  left_join(data, by = "ID") %>%
  select(- date)

# 5c. Now "data" is an individual case-crossover dataset. Where caco
#     variable indicate case day (caco = 1) and control days (caCo = 2)

head(data)

#------------------------------------------------------------
# 6. For each case and control sampling date, compute the number of days
#   since the 2013-12-31. This is done to combine cases and control
#   periods with time-series of meteorological data that starts
#   on 2014-01-01.
#------------------------------------------------------------


ref_date <- as.Date("2013-12-31")
data$sampling_date <- as.Date(data$sampling_date)
data$index <- as.numeric(data$sampling_date - ref_date)

# -------------------------------------------------------------
# 7a. Load Whole history of Temperature Exposure for Each NUTS_ID.
#--------------------------------------------------------------

data_tg <- read.csv(file = "data/4.temperatures.csv")
class(data_tg)
dim(data_tg) # 4 NUTS3

#------------------------------------------------------------
# 8a. Merge "data" and "data_temp" into a temporary file
#   ! Remeber that "data_temp" contains all exposures
#   ! at daily level from 1.1.2014 to 31.12.2022
#------------------------------------------------------------

temp_data <- merge(data, data_tg, by = "NUTS_ID", all.x = T)
names(temp_data)
dim(temp_data)

#------------------------------------------------------------
# 9a. Subset exposure history in the 8 weeks (56 days) preceding
#    the case or control sampling date to a list.
#------------------------------------------------------------

# Find where exposure matrix starts 
col <- which(grepl("^X.+", names(temp_data)))[1] #column 6
start <- col - 1 #starting column

# Crop the dataset with exposure history of interest for each case and control

df <- data.frame(matrix(ncol = start + 56, nrow = 0))
names(df) <- c(names(temp_data[1:start]), paste0("tg_lag", c(1:56)))
for (i in 1:dim(temp_data)[1]) {
  df[i,] <- temp_data[i, c(1:start, (start + temp_data$index[i] - 1):(start + temp_data$index[i] - 56))]
}

#------------------------------------------------------------
# 10a. Re-express measurment unit to C° (x-273)
#------------------------------------------------------------

df[, which(substr(names(df), 1, 2) == "tg")] <- apply(df[, which(substr(names(df), 1, 2) == "tg")], FUN = as.numeric, MARGIN = 2)
df[, which(substr(names(df), 1, 2) == "tg")] <- apply(df[, which(substr(names(df), 1, 2) == "tg")], function(x) {
  x - 273
  }, MARGIN = 2)

#------------------------------------------------------------
# 7b. Load Whole history of Precipitation Exposure for Each NUTS_ID
#------------------------------------------------------------
data_prcp <- read.csv(file = "data/5.precipitation.csv")
dim(data_prcp)

#------------------------------------------------------------
# 8b. Merge "data" and "data_prep" into a temporary file 2
#   ! Remeber that "data_prep" contains all exposures
#   ! at daily level from 1.1.2013 to 31.12.2018
#------------------------------------------------------------

temp_data2 <- merge(data, data_prcp, by = "NUTS_ID", all.x = T)

#------------------------------------------------------------
# 9b. Subset Exposure history in the 8 weeks (56 days) preceding
#    case or control sampling dates to a list.
#------------------------------------------------------------

# Find where exposure matrix starts 
col <- which(grepl("^X.+", names(temp_data2)))[1] #column 8
start <- col - 1
# Crop the dataset with exposure history of interest for each case and control
df2 <- data.frame(matrix(ncol = start + 56, nrow = 0))
names(df2) <- c(names(temp_data2[1:start]), paste0("pr_lag", c(1:56)))
for (i in 1:dim(temp_data)[1]) {
  df2[i,] <- temp_data2[i, c(1:start, (start + temp_data2$index[i] - 1):(start + temp_data2$index[i] - 56))]
}

#------------------------------------------------------------
# 10b. Re-express measurment unit to mm (x*60*60*24*1000)
#------------------------------------------------------------

df2[, which(substr(names(df2), 1, 2) == "pr")] <- apply(df2[, which(substr(names(df2), 1, 2) == "pr")], FUN = as.numeric, MARGIN = 2)
df2[, which(substr(names(df2), 1, 2) == "pr")] <- apply(df2[, which(substr(names(df2), 1, 2) == "pr")], function(x) {
  x * 60 * 60 * 24 * 1000
}, MARGIN = 2)

#------------------------------------------------------------
# 12. Merge df1(case_crossover_temp) e df2 (case_crossover_prcp).
#------------------------------------------------------------

CC_data <- merge(df, df2, by = c(names(df)[1:start]))
dim(CC_data)
ftable(CC_data$caco)
head(CC_data)

#------------------------------------------------------------
# 13. Save the final case-crossover dataset for analyses.
#------------------------------------------------------------

write.csv(CC_data,
  file = "data/7.case_cross_data.csv",
  row.names = FALSE
)

